import { getStore, init } from 'node-cache-redis';
import RedisStore from 'node-cache-redis/dist/src/RedisStore';
import { CacheManager } from './interfaces/cache-manager.interface';

export class RedisCacheManager implements CacheManager {
  private static store: RedisStore;
  public static getStore(): RedisStore {
    if (!RedisCacheManager.store) {
      new RedisCacheManager().init({ name: 'RedisCache', redisOptions: { url: process.env.REDIS_URL } });
      RedisCacheManager.store = getStore();
    }
    return RedisCacheManager.store;
  }
  init(options: any): void {
    init(options);
  }
  async set(key: string, value: any, ttlInSeconds?: number): Promise<string> {
    return await RedisCacheManager.getStore().set(key, value, ttlInSeconds);
  }
  async getset(key: string, value: any, ttlInSeconds?: number): Promise<any> {
    return await RedisCacheManager.getStore().getset(key, value, ttlInSeconds);
  }
  async get(key: string): Promise<any> {
    return await RedisCacheManager.getStore().get(key);
  }
  async del(keys?: string[]): Promise<number> {
    return await RedisCacheManager.getStore().del(keys);
  }
  async deleteAll(pattern?: string): Promise<number> {
    return await RedisCacheManager.getStore().deleteAll(pattern);
  }
}
