import { CacheManager } from '@caches/interfaces/cache-manager.interface';
import { RedisCacheManager } from '@caches/redis-manager.cache';

/* eslint-disable @typescript-eslint/no-non-null-assertion */
function cacheFactory(metadataKey: Symbol, source: 'create' | 'update' | 'delete', config?: CacheConfig) {
  return function (target: any, propertyName: string, descriptor: any) {
    const fn = descriptor.value!;
    descriptor.value = async function (...args: any[]) {
      let res = args[1];
      if (args.length == 4) {
        res = args[2];
      }
      if (args.length == 5) {
        res = args[3];
      }

      let ttl = Number(process.env.CACHE_TTL || 120);
      if (config && config.ttl) {
        ttl = config.ttl;
      }
      const cacheManager: CacheManager = new RedisCacheManager();
      const key = 'cache:' + res.originalUrl;
      const dataString: any = await cacheManager.get(key);
      let returnObj;
      switch (source) {
        case 'create':
          const noCache = res.originalUrl.includes(`no-cache=true`);
          if (dataString && !noCache) {
            if (typeof dataString != 'object') {
              returnObj = JSON.parse(dataString);
            } else {
              returnObj = dataString;
            }
          } else {
            returnObj = await fn.apply(this, args);
            cacheManager.set(key, JSON.stringify(returnObj), ttl);
          }
          break;
        case 'update':
          returnObj = await fn.apply(this, args);
          cacheManager.set(key, JSON.stringify(returnObj), ttl);
          break;
        case 'delete':
          returnObj = await fn.apply(this, args);
          cacheManager.del([key]);
          break;
        default:
          break;
      }
      return returnObj;
    };
    return descriptor;
  };
}

export const Cache = (config?: CacheConfig) => cacheFactory(Symbol('create'), 'create', config);
export const CacheUpdate = (config?: CacheConfig) => cacheFactory(Symbol('update'), 'update', config);
export const CacheDelete = (config?: CacheConfig) => cacheFactory(Symbol('delete'), 'delete', config);

export interface CacheConfig {
  key?: string;
  ttl?: number;
}
