import AppError from '@utils/class/app-error.utils';
import {
  registerDecorator,
  ValidationArguments,
  ValidationOptions,
  ValidatorConstraint,
  ValidatorConstraintInterface,
} from 'class-validator';
import Container from 'typedi';
import { HandleNotFound } from './functional-exception.decorator';

@ValidatorConstraint({ async: true })
export class ExistsConstraint implements ValidatorConstraintInterface {
  @HandleNotFound()
  async validate(value: number | string, validationArguments?: ValidationArguments) {
    const service: any = validationArguments.constraints[0];
    const columnName = validationArguments.constraints[1] ?? 'id';
    const message = validationArguments.constraints[2] ?? `${columnName} not found`;
    const obj: any = {};
    obj[columnName] = value;
    const sr: any = Container.get(service);
    const result = await sr.findOne(obj);
    if (!result) throw new AppError(400, 'Bad Request', message);
    return true;
  }
}

export function Exists(validationOptions?: IClassValidation) {
  return function (object: Object, propertyName: string) {
    registerDecorator({
      target: object.constructor,
      propertyName: propertyName,
      options: validationOptions,
      constraints: [validationOptions.service, validationOptions.column, validationOptions.message],
      validator: ExistsConstraint,
    });
  };
}

export interface IClassValidation extends ValidationOptions {
  column?: string;
  service: any;
}
