import { Service } from 'typedi';
import { HandleNotFound } from '@decorators/functional-exception.decorator';
import { BaseService } from './base/base.service';
import { User } from '@utils/interfaces/user.interface';
import { UserRepository } from '@repositories/knex/user.repository';
import OtpHelper from '@utils/class/otp-helper.utils';
import { Users } from '@models/user.model';
import { mobileSanity } from '@utils/functions/helper-function.util';
import { eventDispatcher } from '@events/event';
import { UserDto } from '@dtos/user.dto';

@Service()
export class UserService extends BaseService<User, UserDto> {
  constructor() {
    super(new UserRepository(Users));
  }
  @HandleNotFound('User Not Found')
  async getUserByMobile(mobile: string) {
    const mob = mobileSanity(mobile);
    const user: User = await this.repository.findOne(
      { mobile: mob },
      '[badges,coupons,timelines,event_participants.event]',
    );
    return user;
  }

  async sendOtp(mobile: string) {
    const user = await this.getUserByMobile(mobile);
    user.otp = OtpHelper.generateOtp();
    await this.repository.update(user.id, user);
    return true;
  }

  async createOrUpdateUser(userDto: UserDto) {
    const mobile = mobileSanity(userDto.mobile);
    userDto.mobile = mobile;
    const user = await this.repository.findOne({ mobile }, '[badges,coupons,timelines,event_participants.event]');
    console.log('Logging User');
    console.log(user);
    if (user) {
      return user;
    }
    userDto.status = true;
    const newUser = await this.insertGraph(userDto, '[badges,coupons,timelines,event_participants.event]');
    eventDispatcher.dispatch('on-user-create', newUser);
    return newUser;
  }
}
