import AWS, { S3 } from 'aws-sdk';
import { awsS3config, s3bucket } from '@config/aws-sdk.config';

export class StorageHelper {
  bucket = s3bucket;
  storage: S3;
  signedUrlExpireSeconds = 60 * 5;

  constructor() {
    this.storage = new AWS.S3(awsS3config);
  }

  async saveBuffer(buffer: any, fileName: string, dest: string, params: object = {}) {
    const s3Object = {
      Bucket: this.bucket,
      Key: dest + '/' + fileName,
      Body: buffer,
      ...params,
    };
    return await this.storage.upload(s3Object).promise();
  }

  async deleteBuffer(file: string) {
    const s3Object = {
      Bucket: this.bucket,
      Key: file,
    };
    return await this.storage.deleteObject(s3Object).promise();
  }

  async getBuffer(file: string) {
    return await this.storage
      .getObject({
        Bucket: this.bucket,
        Key: file,
      })
      .promise();
  }

  getSignedUrl(file: string) {
    return this.storage.getSignedUrl('getObject', {
      Bucket: this.bucket,
      Key: file,
      Expires: this.signedUrlExpireSeconds,
    });
  }
}
