import AppError from '@utils/class/app-error.utils';
import { plainToInstance } from 'class-transformer';
import { validate, ValidationError } from 'class-validator';
import { json2array } from './helper-function.util';

export async function validateClass<Type>(
  type: any,
  data: any,
  skipMissingProperties = false,
  whitelist = false,
  forbidNonWhitelisted = true,
): Promise<Type> {
  const errors: ValidationError[] = await validate(plainToInstance(type, data), {
    skipMissingProperties,
    whitelist,
    forbidNonWhitelisted,
  });
  if (errors.length > 0) {
    const collect: any = [];
    errors.forEach((error: any) =>
      collect.push({
        key: error.property,
        message: json2array(error.constraints),
      }),
    );
    const message = collect;
    throw new AppError(400, 'Validation Failed', message);
  }
  return data;
}
