/* eslint-disable prettier/prettier */
import { StorageHelper } from '@utils/class/storage-helper.utils';
import { StorageConfig } from '@utils/interfaces/base/storage-config.interface';
import handlebars from 'handlebars';
import { PdfGeneratorDTO } from '../dtos/pdf-generator.dto';
const fs = require('fs');
const path = require('path');
const puppeteer = require('puppeteer');
export class PdfGenerator<T> {
  template: any;
  pdfGeneratorDTO: PdfGeneratorDTO<T>;
  constructor(pdfGeneratorDTO: PdfGeneratorDTO<T>) {
    this.pdfGeneratorDTO = pdfGeneratorDTO;
  }

  handlebars() {
    handlebars.registerHelper('inc', function (value: any) {
      return parseInt(value) + 1;
    });
    handlebars.registerHelper('distanceFixed', function (distance: any) {
      if (Number(distance) != NaN) {
        distance = Number(distance);
        return distance.toFixed(2);
      } else {
        return distance;
      }
    });
    // ref from https://gist.github.com/pheuter/3515945
    handlebars.registerHelper('compare', function (lvalue, operator, rvalue, options) {
      const operators: any = {
        '==': function (l: any, r: any) {
          return l == r;
        },
        '===': function (l: any, r: any) {
          return l === r;
        },
        '!=': function (l: any, r: any) {
          return l != r;
        },
        '!==': function (l: any, r: any) {
          return l !== r;
        },
        '<': function (l: any, r: any) {
          return l < r;
        },
        '>': function (l: any, r: any) {
          return l > r;
        },
        '<=': function (l: any, r: any) {
          return l <= r;
        },
        '>=': function (l: any, r: any) {
          return l >= r;
        },
        typeof: function (l: any, r: any) {
          return typeof l == r;
        },
      };
      let result: any;

      if (arguments.length < 3) {
        throw new Error("Handlerbars Helper 'compare' needs 2 parameters");
      }

      if (options === undefined) {
        options = rvalue;
        rvalue = operator;
        operator = '===';
      }

      if (!operators[operator]) {
        throw new Error("Handlerbars Helper 'compare' doesn't know the operator " + operator);
      }

      // eslint-disable-next-line prefer-const
      result = operators[operator](lvalue, rvalue);

      if (result) {
        return options.fn(this);
      } else {
        return options.inverse(this);
      }
    });
    return handlebars;
  }
  getOptions() {
    return Boolean(this.pdfGeneratorDTO.options)
      ? this.pdfGeneratorDTO.options
      : {
        format: 'A4',
        headerTemplate: '<p></p>',
        footerTemplate: '<p></p>',
        displayHeaderFooter: false,
        margin: {
          top: '40px',
          bottom: '100px',
        },
        printBackground: true,
        path: this.pdfGeneratorDTO.filename,
      };
  }
  async generateHtml() {
    const templateFile = this.getTemplate();
    const template = this.handlebars().compile(templateFile);
    return encodeURIComponent(template(this.pdfGeneratorDTO.data));
  }
  async getPdf() {
    const browser = await puppeteer.launch({
      args: ['--no-sandbox'],
      headless: true,
    });
    const page = await browser.newPage();
    const html = await this.generateHtml();
    await page.goto(`data:text/html;charset=UTF-8,${html}`, {
      waitUntil: 'networkidle0',
    });
    const pdf = await page.pdf(this.getOptions());
    await browser.close();
    return pdf;
  }

  async saveToS3(storageConfig: StorageConfig) {
    storageConfig.fileName = storageConfig.fileName ?? this.pdfGeneratorDTO.filename;
    return await new StorageHelper().saveBuffer(
      await this.getPdf(),
      storageConfig.fileName,
      storageConfig.dest,
    );
  }
  getTemplate() {
    return fs.readFileSync(
      path.join(process.cwd(), `/dist/app/utils/pdfGenerator/templates/${this.pdfGeneratorDTO.path}`),
      'utf8',
    );
  }
}
